/* 
 * Kosmo - Sistema Abierto de Informacin Geogrfica
 * Kosmo - Open Geographical Information System
 *
 * http://www.saig.es
 * (C) 2010, SAIG S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * For more information, contact:
 * 
 * Sistemas Abiertos de Informacin Geogrfica, S.L.
 * Avnda. Repblica Argentina, 28
 * Edificio Domocenter Planta 2 Oficina 7
 * C.P.: 41930 - Bormujos (Sevilla)
 * Espaa / Spain
 *
 * Telfono / Phone Number
 * +34 954 788876
 * 
 * Correo electrnico / Email
 * info@saig.es
 *
 */
package es.kosmo.desktop.plugins.help;

import java.net.MalformedURLException;
import java.net.URL;

import org.saig.jump.lang.I18N;

import com.vividsolutions.jump.workbench.JUMPWorkbench;
import com.vividsolutions.jump.workbench.WorkbenchContext;
import com.vividsolutions.jump.workbench.plugin.AbstractPlugIn;
import com.vividsolutions.jump.workbench.plugin.EnableCheck;
import com.vividsolutions.jump.workbench.plugin.EnableCheckFactory;
import com.vividsolutions.jump.workbench.plugin.MultiEnableCheck;
import com.vividsolutions.jump.workbench.plugin.PlugInContext;

import es.kosmo.desktop.utils.DesktopUtils;

/**
 * Base class for opening a web site from a plugin
 * <p>
 * </p>
 * 
 * @author Sergio Ba&ntilde;os Calvo - sbc@saig.es
 * @since 2.0 [Giuseppe Aruta 2017_11_28] Adapted to OpenJUMP Version 20171022
 *        snapshot rev.5522
 */
public abstract class AbstractOpenWebSitePlugIn extends AbstractPlugIn {

    @Override
    public boolean execute(PlugInContext context) throws Exception {
        if (isValid(getURL())) {
            DesktopUtils.browse(getURL());
        } else {
            JUMPWorkbench
                    .getInstance()
                    .getFrame()
                    .warnUser(
                            I18N.getMessage(
                                    "es.kosmo.desktop.plugins.help.AbstractOpenWebSitePlugIn.The-URL-{0}-is-not-valid", //$NON-NLS-1$  
                                    new Object[] { getURL() }));
        }

        return true;
    }

    /**
     * Returns the URL to open in the system configured browser
     * 
     * @return
     * @throws MalformedURLException
     */
    public abstract URL getURL() throws MalformedURLException;

    /**
     * Checks that the URL is correct
     * 
     * @param url
     * @return
     */
    protected boolean isValid(URL url) {
        return true;
    }

    /**
     * Devuelve la comprobacion que debe cumplirse para poder activar el plugin
     * 
     * @param workbenchContext
     *            Contexto de la aplicacion
     * @return EnableCheck - Comprobacion que debe cumplir el plugin para poder
     *         activarse
     */
    public static EnableCheck createEnableCheck(
            WorkbenchContext workbenchContext) {
        EnableCheckFactory checkFactory = new EnableCheckFactory(
                workbenchContext);

        return new MultiEnableCheck().add(checkFactory
                .createWindowWithLayerViewPanelMustBeActiveCheck())
        // .add(checkFactory.createAtLeastNLayersMustBeEditableCheck(1))
        ;
    }

}
